-- https://github.com/OxfordAbstracts/purescript-graphql-client/pull/68/files#diff-404493334c4108c2411bd1767059be2e94295df5742d14d58116667dc5e80dd7R16-R19

module Gargantext.Utils.GraphQL
  -- | Get the graphql fields from a data type
  ( class GetFieldsStandard
  , PropGetFieldsStandard
  , getFieldsStandard
  , unwrapGraphQLResult
  ) where

import Prelude

import Data.Array as A
import Data.Bifunctor (rmap)
import Data.Either (Either(..), note)
import Data.HeytingAlgebra (class HeytingAlgebraRecord, tt)
import Data.Maybe (Maybe)
import Data.Tuple (Tuple(..))
import Gargantext.Config.REST (RESTError(CustomError))
import Heterogeneous.Mapping (class HMap, class Mapping, hmap)
import Prim.RowList (class RowToList)
import Type.Proxy (Proxy(..))

class GetFieldsStandard :: forall k. k -> Type -> Constraint
class GetFieldsStandard t fields | t -> fields where
  getFieldsStandard :: Proxy t -> fields

data PropGetFieldsStandard = PropGetFieldsStandard

data PropToProxy = PropToProxy

instance getFieldsStandardRecord ::
  ( RowToList r t
  , HeytingAlgebraRecord t r r
  , HMap PropGetFieldsStandard { | r } fields
  , HMap PropGetFieldsStandard { | input } { | r }
  ) =>
  GetFieldsStandard { | input } fields where
  getFieldsStandard _ = recordGetFieldsStandard (tt :: { | r })
else instance getFieldsStandardMaybe :: GetFieldsStandard a fields => GetFieldsStandard (Maybe a) fields where
  getFieldsStandard _ = getFieldsStandard (Proxy :: _ a)
else instance getFieldsStandardArray :: GetFieldsStandard a fields => GetFieldsStandard (Array a) fields where
  getFieldsStandard _ = getFieldsStandard (Proxy :: _ a)
else instance getFieldsStandardLeaf :: GetFieldsStandard a Unit where
  getFieldsStandard _ = unit

instance propToProxy ::
  Mapping PropToProxy t (Proxy t) where
  mapping PropToProxy _ = Proxy

recordGetFieldsStandard
  :: forall t fields
   . HMap PropGetFieldsStandard ({ | t }) fields
  => { | t }
  -> fields
recordGetFieldsStandard = hmap PropGetFieldsStandard

instance propGetFieldsStandard ::
  ( GetFieldsStandard t fields
  ) =>
  Mapping PropGetFieldsStandard t fields where
  mapping PropGetFieldsStandard _ = getFieldsStandard (Proxy :: _ t)

-- | GraphQL returns an Either Error a result type and one usually has to get the head of a
unwrapGraphQLResult
  :: forall a params
   . (Show params)
  => Tuple String params
  -> Either RESTError (Array a)
  -> Either RESTError a
unwrapGraphQLResult _ (Left err) = Left err
unwrapGraphQLResult (Tuple name params) (Right res) =
  note (CustomError $ "[unwrapGraphQLResult] " <> name <> " with params " <> show params <> " not found")
    (A.head res)
